-------------------------------------------------------------------------------
--
-- Copyright (c) 2014, 2015 Ales Machat (Garkin)
--
-- Permission is hereby granted, free of charge, to any person
-- obtaining a copy of this software and associated documentation
-- files (the "Software"), to deal in the Software without
-- restriction, including without limitation the rights to use,
-- copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the
-- Software is furnished to do so, subject to the following
-- conditions:
--
-- The above copyright notice and this permission notice shall be
-- included in all copies or substantial portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-- EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
-- OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
-- NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
-- HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-- FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
-- OTHER DEALINGS IN THE SOFTWARE.
--
-------------------------------------------------------------------------------
--
-- DISCLAIMER:
--
-- This Add-on is not created by, affiliated with or sponsored by ZeniMax
-- Media Inc. or its affiliates. The Elder Scrolls® and related logos are
-- registered trademarks or trademarks of ZeniMax Media Inc. in the United
-- States and/or other countries. All rights reserved.
--
-- You can read the full terms at:
-- https://account.elderscrollsonline.com/add-on-terms
--
-------------------------------------------------------------------------------

local ADDON_NAME = "ShieldInfo"
local EM = GetEventManager()
local WM = GetWindowManager()
local SV, UI
local current = 0
local maximum = 0


local function SetupUI(state)
   UI:SetMouseEnabled(state)
   UI:SetMovable(state)

   UI.bd:SetHidden(not state)
   UI:SetHidden(not (state or (current > 0)))

   if current > 0 then
      UI.label:SetText(("%d/%d"):format(current, maximum))
   else
      UI.label:SetText("current/max")
   end

   if not state then
      local valid, point, target, relPoint, x, y = UI:GetAnchor(0)

      if valid then
         SV.Anchor = { p = point, t = target:GetName(), r = relPoint, x = x, y = y }
      end 
   end
end

local function OnMouseUp(self, button, upInside)
   if upInside then
      if button == 2 then
         ClearMenu()
         AddMenuItem("Save & Hide", function() SetupUI(false) end)
         ShowMenu(self)
      end
   end
end

local function UnitAttributeVisual(evt, unitTag, unitAttributeVisual, statType, attributeType, powerType, value1, value2, value3, value4)
   if unitTag == "player" and unitAttributeVisual == ATTRIBUTE_VISUAL_POWER_SHIELDING and attributeType == ATTRIBUTE_HEALTH then
      if evt == EVENT_UNIT_ATTRIBUTE_VISUAL_UPDATED then
         current = current + (value2 - value1)
         maximum = maximum + (value4 - value3)
      elseif evt == EVENT_UNIT_ATTRIBUTE_VISUAL_ADDED then
         current = current + value1
         maximum = maximum + value2
      elseif evt == EVENT_UNIT_ATTRIBUTE_VISUAL_REMOVED then
         current = current - value1
         maximum = maximum - value2
      end
   end

   if current > 0 then
      UI.label:SetText(("%d/%d"):format(current, maximum))
      UI:SetHidden(false)
   else
      UI:SetHidden(true)
   end
end

local function OnUnitDeath(evt, unitTag, isDead)
   if isDead and unitTag == "player" then
      current = 0
      maximum = 0
      UI:SetHidden(true)
   end
end

local function OnLoaded(evt, name)
   if name == ADDON_NAME then
      EM:UnregisterForEvent(name, evt)

      SV = ZO_SavedVars:New("SHIELD_INFO_SV", 1, { Anchor = { p = BOTTOM, t = "GuiRoot", r = CENTER, x = 0, y = -100 }, firstTime = true })

      UI = WM:CreateControlFromVirtual(nil, GuiRoot, "ShieldInfo_Template")

      UI:SetHandler("OnMouseUp", OnMouseUp)

      if SV.firstTime then
         SV.firstTime = false
         d("=== Shield Info ===")
         d("Adjust ShieldInfo position and then right-click to hide window and save its position.")
         d("Type /si to unlock ShieldInfo window again.")
         SetupUI(true)
      end

      UI:SetAnchor(SV.Anchor.p, _G[SV.Anchor.t], SV.Anchor.r, SV.Anchor.x, SV.Anchor.y)

      UI:RegisterForEvent(EVENT_UNIT_ATTRIBUTE_VISUAL_ADDED, UnitAttributeVisual)
      UI:RegisterForEvent(EVENT_UNIT_ATTRIBUTE_VISUAL_REMOVED, UnitAttributeVisual)
      UI:RegisterForEvent(EVENT_UNIT_ATTRIBUTE_VISUAL_UPDATED, UnitAttributeVisual)
      UI:RegisterForEvent(EVENT_UNIT_DEATH_STATE_CHANGED, OnUnitDeath)

      SLASH_COMMANDS["/si"] = function() SetupUI(true) end
   end
end

EM:RegisterForEvent(ADDON_NAME, EVENT_ADD_ON_LOADED, OnLoaded)